-- ======================================================================
-- == VOICE RECOGNITION & GEMINI CORRECTION SCRIPT (v25.1 - New Prompt) ==
-- ======================================================================
-- == Base: User's v25 script.                                         ==
-- == Modification: Updated Gemini prompt for Unconditional English    ==
-- ==               Translation, Grammar Correction, and Emoji Addition.==
-- == Goal: Translate All to English + Grammar Fix + Add Emoji + Strict Output. ==
-- ======================================================================

-- == Step 0: Initial Setup and Bindings ==
local scriptStartTime = os.time()

this = activity or service
if not this then return false end

local function my_speak_service(message) -- Using function name from user's working base
  pcall(function()
    if this and this.service and this.service.asyncSpeak then
      this.service.asyncSpeak(tostring(message or "Walang mensahe"))
    elseif this and this.asyncSpeak then
       this.asyncSpeak(tostring(message or "Walang mensahe"))
    end
  end)
end

if not ((this and this.service and this.service.asyncSpeak) or (this and this.asyncSpeak)) then return false end

-- Bind necessary classes
local Http, SpeechRecognizer, RecognizerIntent, Intent, Handler, ComponentName, Uri, InputMethodManager, View
local ClipboardManager, ClipData, Context

local bindSuccess, bindResult = pcall(function()
  Http = luajava.bindClass("com.androlua.Http")
  SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  Intent = luajava.bindClass("android.content.Intent")
  Handler = luajava.bindClass("android.os.Handler")
  ComponentName = luajava.bindClass("android.content.ComponentName")
  Uri = luajava.bindClass("android.net.Uri")
  InputMethodManager = luajava.bindClass("android.view.inputmethod.InputMethodManager")
  View = luajava.bindClass("android.view.View")
  ClipboardManager = luajava.bindClass("android.content.ClipboardManager")
  ClipData = luajava.bindClass("android.content.ClipData")
  Context = luajava.bindClass("android.content.Context")
  cjson = require "cjson"
end)

if not bindSuccess then
  my_speak_service("Fatal Error: Hindi ma-bind ang mga klase.")
  return false
end

-- Global variables
local currentRecognizer = nil
local isListening = false
local geminiApiKey = os.getenv("GEMINI_API_KEY") or "AIzaSyArz06zWTwsG6LVd9yD0fZ0gCvcHOm9C9k" -- THIS LINE REMAINS UNCHANGED

-- === GEMINI API FUNCTION ===
function correctGrammarWithGemini(rawText)
  if not geminiApiKey or geminiApiKey == "ILAGAY_ANG_IYONG_API_KEY_DITO" then my_speak_service("Error: Gemini API Key is missing..."); return end
  if not rawText or rawText == "" then return end

  local modelName = "gemini-1.5-flash-latest"
  local geminiUrl = "https://generativelanguage.googleapis.com/v1beta/models/" .. modelName .. ":generateContent?key=" .. geminiApiKey

  -- ***************************************************************************************
  -- *** START OF MODIFIED SECTION: New Gemini Prompt as per your latest instructions ***
  -- ***************************************************************************************
  local correctionPrompt = string.format([[
**STRICT INSTRUCTIONS FOR AI:**

1.  **PRIMARY GOAL:** Your sole task is to process the user's input text according to the rules below.

2.  **TRANSLATION TO ENGLISH:**
    * You **MUST** translate the **ENTIRE** input text into **STANDARD, FLUENT, and GRAMMATICALLY PERFECT ENGLISH**.
    * Ensure the translation is accurate and faithfully preserves the original meaning and nuance of the input text.

3.  **EMOJI ADDITION:**
    * After the translated English text, you **MUST** append **EXACTLY ONE (1)** emoji.
    * This emoji **MUST** be highly relevant to the context, sentiment, or main subject of the translated English text.
    * Choose a common, universally understood emoji.

4.  **OUTPUT FORMATTING (CRUCIAL):**
    * The final output **MUST ONLY** consist of the translated English text followed immediately by the single emoji.
    * **ABSOLUTELY NO** other text, explanations, apologies, greetings, or any form of conversational filler before or after the translated text and emoji.
    * **DO NOT** use any markdown formatting (e.g., bold, italics, backticks) in your output.
    * **DO NOT** add quotation marks around the output unless they were explicitly part of the original meaning intended for translation.

5.  **BEHAVIORAL CONSTRAINTS (STRICTLY ENFORCED):**
    * You **MUST NOT** ask clarifying questions to the user.
    * You **MUST NOT** offer alternatives or suggestions.
    * You **MUST NOT** explain your reasoning, the translation process, or your emoji choice.
    * You **MUST NOT** alter, add to, or remove from the core meaning of the user's original input text during translation, other than what is required for accurate translation and grammar correction in English. The emoji is the only specifically requested additive element to the translated text itself.
    * If the input is nonsensical or too short to derive meaning, output only a single neutral face emoji (e.g., 😶).

**User's Input Text:**
"%s"
]], tostring(rawText)) -- Ensure rawText is string
  -- ***************************************************************************************
  -- *** END OF MODIFIED SECTION: New Gemini Prompt                                       ***
  -- ***************************************************************************************

  local requestBodyTable = { contents = { { parts = { { text = correctionPrompt } } } }, generationConfig = { temperature = 0.2, maxOutputTokens = 1000 } } -- Temperature slightly adjusted for creativity in emoji if needed, but still low.
  local successEncode, jsonResult = pcall(cjson.encode, requestBodyTable)
  if not successEncode then my_speak_service("Error encoding request."); return end
  local jsonRequestBodyString = jsonResult
  local headers = { ["Content-Type"] = "application/json" }

  local successHttpPostCall, errorHttpPostCall = pcall(function()
    Http.post(geminiUrl, jsonRequestBodyString, headers, function(status, responseBody)
      if status == 200 then
        local responseData = nil
        local successDecode, decodeResult = pcall(cjson.decode, responseBody or "")
        if successDecode and decodeResult then
          responseData = decodeResult
          local extractedText = nil
          local pcallSuccessExtract, pcallResultExtract = pcall(function()
             if responseData.candidates and responseData.candidates[1] and responseData.candidates[1].content and responseData.candidates[1].content.parts and responseData.candidates[1].content.parts[1] and responseData.candidates[1].content.parts[1].text then extractedText = responseData.candidates[1].content.parts[1].text
             elseif responseData.candidates and responseData.candidates[1] and responseData.candidates[1].finishReason == "SAFETY" then extractedText = "Gemini blocked: Safety. 🛡️"; my_speak_service(extractedText) -- Added emoji for consistency
             elseif responseData.promptFeedback and responseData.promptFeedback.blockReason then extractedText = "Gemini blocked: Prompt issue. 🚫"; my_speak_service(extractedText) -- Added emoji for consistency
             else extractedText = nil; my_speak_service("Hindi maintindihan ang response structure. 😕") end -- Added emoji
          end)

          if pcallSuccessExtract and extractedText and not string.find(extractedText, "Gemini blocked:") then -- Check if not a blocked message before processing
             local cleanedText = extractedText:gsub("^[%s\n]*(.-)%s*$", "%1") -- Trim whitespace
             my_speak_service("Resulta: " .. cleanedText) -- Speak the result with emoji

             local successAppend = pcall(function()
               local editText = this.service.getEditText()
               if editText then
                  local currentText = ""; local successGetText, textVal = pcall(function() return editText.getText().toString() end); if successGetText and textVal then currentText = textVal end
                  local newText = nil
                  if currentText == "" then newText = cleanedText else
                      if string.sub(currentText, -1) == " " then newText = currentText .. cleanedText
                      else newText = currentText .. " " .. cleanedText end
                  end
                  local successSetText, setError = pcall(this.service.setText, editText, newText)
                  if successSetText then
                     pcall(function() editText.setSelection(string.len(newText)) end)
                  else
                      my_speak_service("Error updating text field. Nasa clipboard na lang. 📋") -- Added emoji
                      pcall(function() if this and Context and ClipboardManager and ClipData then local clipboardMgr = this.getSystemService(Context.CLIPBOARD_SERVICE); if clipboardMgr then local clipdata = ClipData.newPlainText("corrected_text", cleanedText); clipboardMgr.setPrimaryClip(clipdata); end end end)
                  end
               else
                  my_speak_service("Error: Cannot find focused text field. Nasa clipboard na lang. 📋") -- Added emoji
                  pcall(function() if this and Context and ClipboardManager and ClipData then local clipboardMgr = this.getSystemService(Context.CLIPBOARD_SERVICE); if clipboardMgr then local clipdata = ClipData.newPlainText("corrected_text", cleanedText); clipboardMgr.setPrimaryClip(clipdata); end end end)
               end
             end)
             if not successAppend then my_speak_service("Error occurred during text appending logic. ⚙️") end -- Added emoji
          elseif not extractedText and not string.find(extractedText or "", "Gemini blocked:") then -- Ensure it's not a block message already spoken
            my_speak_service("Error extracting text from Gemini response. 📄")  -- Added emoji
          end
        else my_speak_service("Error decoding Gemini JSON response. 💔") end -- Added emoji
      else
        my_speak_service("Gemini API Error. Status: " .. (status or "unknown") .. " 🌐") -- Added emoji
      end
    end)
  end)
  if not successHttpPostCall then
     my_speak_service("Critical Error: Failed to initiate Http post call. ☎️") -- Added emoji
  end
end

-- === SPEECH RECOGNITION FUNCTIONS === (UNCHANGED)
function stopListening()
  if isListening and currentRecognizer then
    isListening = false
    local stopSuccess, stopResult = pcall(function()
      currentRecognizer.stopListening()
      currentRecognizer.cancel()
      currentRecognizer.destroy()
    end)
    currentRecognizer = nil
    if stopSuccess then
      my_speak_service("Huminto sa pakikinig. 🛑") -- Added emoji
    else
      my_speak_service("Nagka-error sa paghinto. 💥") -- Added emoji
    end
  end
end

function startVoiceGrammarCorrection()
    if isListening then
      my_speak_service("Nakikinig na. 🎤") -- Added emoji
      return
    end

    isListening = true
    local speechNotificationSent = false
    local recognizer = nil
    local intent = nil
    local errorOccurred = false

    local targetPackage = "com.google.android.tts"
    local targetClass = "com.google.android.apps.speech.tts.googletts.service.GoogleTTSRecognitionService"
    local compName = nil

    local createCompSuccess, createCompResult = pcall(function() if ComponentName then compName = ComponentName(targetPackage, targetClass) else error("ComponentName not bound") end end)

    if createCompSuccess and compName then
       local createSpecificSuccess, createSpecificResult = pcall(function() if this and SpeechRecognizer then recognizer = SpeechRecognizer.createSpeechRecognizer(this, compName) else error("'this' or SpeechRecognizer not available") end end);
       if not createSpecificSuccess or recognizer == nil then recognizer = nil end
    end

    if recognizer == nil then
       local createDefaultSuccess, createDefaultResult = pcall(function() if this and SpeechRecognizer then recognizer = SpeechRecognizer.createSpeechRecognizer(this) else error("'this' or SpeechRecognizer not available") end; if recognizer == nil then error("createSpeechRecognizer(default) returned nil") end end);
       if not createDefaultSuccess or recognizer == nil then my_speak_service("Fatal Error: Hindi makagawa ng recognizer (default). 🛠️"); errorOccurred = true end -- Added emoji
    end

    if errorOccurred or recognizer == nil then my_speak_service("Huminto dahil sa error sa recognizer setup. ⚙️"); isListening = false; return end -- Added emoji
    currentRecognizer = recognizer

    local setupIntentSuccess, setupIntentResult = pcall(function()
        if Intent and RecognizerIntent then
           intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
           intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
           intent.putExtra(RecognizerIntent.EXTRA_PROMPT, "Magsalita ka...")
           intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "fil-PH")
           intent.putExtra(RecognizerIntent.EXTRA_SPEECH_INPUT_MINIMUM_LENGTH_MILLIS, 500)
           intent.putExtra(RecognizerIntent.EXTRA_SPEECH_INPUT_COMPLETE_SILENCE_LENGTH_MILLIS, 1000)
           intent.putExtra(RecognizerIntent.EXTRA_PARTIAL_RESULTS, false)
           intent.putExtra(RecognizerIntent.EXTRA_MAX_RESULTS, 1)
        else error("Intent or RecognizerIntent not bound") end; if intent == nil then error("Intent creation failed") end
    end)

    if not setupIntentSuccess or intent == nil then my_speak_service("Error sa pag-setup ng Recognition Intent. ⚙️"); isListening = false; pcall(function() if recognizer then recognizer.destroy() end end); currentRecognizer = nil; return end -- Added emoji

    local listener = nil;
    local listenerSetupSuccess, listenerSetupResult = pcall(function()
      listener = luajava.createProxy("android.speech.RecognitionListener", {
        onReadyForSpeech = function(params) if not speechNotificationSent then my_speak_service("Listening... 👂"); speechNotificationSent = true end end, -- Added emoji
        onBeginningOfSpeech = function() end, onRmsChanged = function(rmsdB) end, onBufferReceived = function(buffer) end, onEndOfSpeech = function() end,
        onResults = function(results)
            isListening = false; local currentRec = currentRecognizer; currentRecognizer = nil; local recognizedText = "Error processing results"
            local pcallSuccess, pcallResult = pcall(function()
               if results == nil then error("Results object is nil") end;
               local matches = results.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION);
               if matches ~= nil and matches.size() > 0 then recognizedText = matches.get(0); if recognizedText == nil or recognizedText == "" then recognizedText = "Walang nakuha na resulta" end
               else recognizedText = "Walang nakuhang tugma" end
            end);
            if pcallSuccess then
               if recognizedText ~= "Walang nakuhang tugma" and recognizedText ~= "Error processing results" and recognizedText ~= "Walang nakuha na resulta" then
                  local handler = Handler(); handler.postDelayed(function() correctGrammarWithGemini(recognizedText) end, 200)
               else my_speak_service("Hindi nakuha ang sinabi. Pakiulit. 🔁") end -- Added emoji
            else my_speak_service("Listener: Error sa pagkuha ng resulta. Pakiulit. 🔁"); end; -- Added emoji
            if currentRec then pcall(function() currentRec.destroy() end) end
        end,
        onError = function(error) isListening = false; local currentRec = currentRecognizer; currentRecognizer = nil; local errorMsg = "Voice recognition error. 🚫"; local errorCode = tonumber(error); -- Added emoji
           if errorCode == SpeechRecognizer.ERROR_NO_MATCH then errorMsg = "Walang narinig. Pakiulit. 🤷" elseif errorCode == SpeechRecognizer.ERROR_SPEECH_TIMEOUT then errorMsg = "Walang nagsalita. Pakiulit. ⏳" elseif errorCode == SpeechRecognizer.ERROR_NETWORK or errorCode == SpeechRecognizer.ERROR_NETWORK_TIMEOUT then errorMsg = "Network error. Check connection. 🌐" elseif errorCode == SpeechRecognizer.ERROR_INSUFFICIENT_PERMISSIONS then errorMsg = "Kailangan ng mic permission. 🎤" else errorMsg = "Recognition error code: "..tostring(error) .. " ⚠️" end; -- Added emojis
           my_speak_service(errorMsg);
           if currentRec then pcall(function() currentRec.destroy() end) end
        end,
        onPartialResults = function(partialResults) end, onEvent = function(eventType, params) end }); if recognizer and listener then recognizer.setRecognitionListener(listener) else error("Recognizer or Listener is nil before setting") end end)

    if not listenerSetupSuccess then my_speak_service("Error sa pag-setup ng listener. 🎧"); isListening = false; pcall(function() if recognizer then recognizer.destroy() end end); currentRecognizer = nil; return end -- Added emoji

    local startSuccess, startResult = pcall(function() if recognizer and intent then recognizer.startListening(intent) else error("Recognizer or Intent is nil before starting") end end)
    if not startSuccess then my_speak_service("Fatal Error starting voice recognition. 💥"); isListening = false; pcall(function() if recognizer then recognizer.destroy() end end); currentRecognizer = nil; end -- Added emoji
end
-- === END OF SPEECH RECOGNITION FUNCTIONS === (UNCHANGED)


-- Button click event (UNCHANGED)
function onButtonClick()
    startVoiceGrammarCorrection()
end

-- === Simulan ang Script === (UNCHANGED)
local autoStart = true
if autoStart then
   startVoiceGrammarCorrection()
else
   my_speak_service("Handa na. Pindutin ang button. 👍") -- Added emoji
end

return true